// This may look like C code, but it's really -*- C++ -*-
/*
 * Copyright (C) 2008 Emweb bv, Herent, Belgium.
 *
 * See the LICENSE file for terms of use.
 */
#ifndef WPOPUP_MENU_ITEM_H_
#define WPOPUP_MENU_ITEM_H_

#include <Wt/WMenuItem.h>

#endif // WPOPUP_MENU_ITEM_H_
