/* additional style rules for IE */

.Wt-dialogcover {
    filter: alpha(opacity=50);
}

/*
 * WItemDelegate
 */
.Wt-delegate-edit input {
    font-size: 9pt;
    padding-top: 0px;
    padding-bottom: 0px;
}

/*
 * WSuggestionPopup
 */
.Wt-suggest .content {
    overflow-x: hidden;
}

/*
 * WAbstractItemView
 */
.Wt-itemview .Wt-header .Wt-label {
    zoom: 1;
}

/*
 * WTableView
 */
.Wt-tableview div {
    white-space: nowrap
}

/*
 * WTree
 */
.Wt-tree div, .Wt-tree span {
    white-space: nowrap;   
}

.Wt-tree .Wt-item {
    zoom: 1;
}

/*
 * WTreeView
 */

/* IE ignores white-space on a table or td */
.Wt-treeview div, .Wt-treeview span {
    white-space: nowrap;
}

body.Wt-ltr .Wt-treeview .Wt-header .Wt-tv-c {
    margin: 0px;
    padding: 0px 0px 0px 7px;
}

body.Wt-rtl .Wt-treeview .Wt-header .Wt-tv-c {
    margin: 0px;
    padding: 0px 7px 0px 0px;
}

.Wt-treeview .Wt-item {
    zoom: 1;
}

.Wt-treeview .Wt-scroll {
    position: absolute;
    overflow-x: auto;
    overflow-y: hidden;
}

.Wt-hrh {
    line-height: 0px;
}