/* additional style rules for ie6 */

/*
 * Wt-outset
 */
.Wt-outset {
    border: 1px solid black;
}

.Wt-outset .Wt-x1, .Wt-outset .Wt-x1a, .Wt-outset .Wt-x2, .Wt-outset .Wt-x2a {
    background: transparent;
}

/*
 * WPanel
 */
.Wt-panel {
    position: static;
}
