.Wt-tableview {
  white-space: nowrap;

  .Wt-header {
    .Wt-label {
      padding-inline: 3px 0px;
    }

    .Wt-tv-no-rh ~ * > * > .Wt-label {
      padding-inline-end: 3px;
    }

    div[style*="text-align:center;"] .Wt-tv-sh ~ .Wt-label {
      padding-inline: 19px 0px;
    }

    .Wt-tv-no-rh ~ div[style*="text-align:center;"] > * > .Wt-tv-sh ~ .Wt-label {
      padding-inline-start: 16px;
    }
  }

  .Wt-tv-contents {
    background-color: $pagination-bg;

    &.Wt-striped {
      div.Wt-tv-c:nth-child(odd),
      a.Wt-tv-c:nth-child(odd) {
        background-color: #f9f9f9;
      }

      div.active:nth-child(odd),
      a.active:nth-child(odd) {
        background-color: #428bca;
      }
    }
  }

  .Wt-tv-contents .Wt-tv-c,
  .Wt-plaintable td,
  .Wt-plaintable th {
    padding: 0 3px;
    overflow: hidden;
    text-overflow: ellipsis;
  }

  .Wt-tv-contents img.icon,
  .Wt-plaintable img.icon,
  .Wt-tv-contents input.icon,
  .Wt-plaintable input.icon {
    margin-top: 0;
    margin-bottom: 2px;
    margin-inline: 0 3px;
    vertical-align: middle;
  }

  .Wt-spacer {
    position: relative;
    overflow: hidden;
  }

  table {
    background-position: left bottom;
  }

  th {
    padding: 0;
  }

  .Wt-tv-br {
    border-inline-end: 1px solid #dddddd;
  }

  .Wt-plaintable .Wt-tv-br {
    border-inline-end: 0;
  }

  .Wt-tv-contents .Wt-tv-c,
  .Wt-plaintable td,
  .Wt-plaintable th {
    border-inline-end: 1px solid #dddddd;
    border-bottom: 1px solid #dddddd;
  }

  .Wt-header {
    border-bottom: 1px solid #dddddd;
  }
}

@include color-mode(dark) {
  .Wt-tableview {
    .Wt-tv-contents {
      &.Wt-striped {
        div.Wt-tv-c:nth-child(odd),
        a.Wt-tv-c:nth-child(odd) {
          background-color: #272B2F;
        }

        div.active:nth-child(odd),
        a.active:nth-child(odd) {
          background-color: #4d94d4;
        }
      }
    }

    .Wt-tv-br,
    .Wt-tv-contents .Wt-tv-c,
    .Wt-plaintable td,
    .Wt-plaintable th {
      border-color: #43474B;
    }

    .Wt-header {
      border-bottom-color: #43474B;
    }
  }
}
