// This may look like C code, but it's really -*- C++ -*-
/*
 * Copyright (C) 2014 Emweb bv, Herent, Belgium.
 *
 * See the LICENSE file for terms of use.
 */
#ifndef WT_DBO_ESCAPE_OSTREAM_H_
#define WT_DBO_ESCAPE_OSTREAM_H_

#include "StringStream.h"

#define WT_DBO_ESCAPEOSTREAM
#include "../../web/EscapeOStream.h"
#undef WT_DBO_ESCAPEOSTREAM

#endif // WT_DBO_ESCAPE_OSTREAM_H_
