body, td, button {
  font-family:arial,sans-serif;
}

.Wt-btn {
  margin-left: 10px;
  margin-right: 10px;
  padding-left: 5px;
  padding-right :5px;
  font-size: x-small;
}

.label {
  font-weight: bold;
  font-size: smaller;
  margin-left: 5px;
  margin-right: 5px;
}

.option {
  cursor: pointer;
  color: blue;
  font-size: smaller;
  text-decoration: underline;
}

.sep {
  padding: 5px;
}

button.default {
  font-weight: bold
}

.lighter {
  background-color: #e0ecff;
}

.darker {
  background-color: #C3D9FF;
}

.status {
  font-size: smaller;
}

.error {
  color: red;
  font-size: smaller;
}

.remove {
  cursor: pointer;
  color: blue;
  font-size: x-small;
  text-decoration: underline;
}

.feedback div {
  margin: 10px;
}

a {
  color: blue;
  font-size: smaller;
  text-decoration: underline;
}