td.linenos { background-color: #f0f0f0; padding-right: 10px; }
span.lineno { background-color: #f0f0f0; padding: 0 5px 0 5px; }
pre { line-height: 125%; }
.highlight .hll { background-color: #ffffcc }
.highlight .c { color: #408080; font-style: italic } /* Comment */
.highlight .err { border: 1px solid #FF0000 } /* Error */
.highlight .k { color: #008000; font-weight: bold } /* Keyword */
.highlight .o { color: #666666 } /* Operator */
.highlight .cm { color: #408080; font-style: italic } /* Comment.Multiline */
.highlight .cp { color: #BC7A00 } /* Comment.Preproc */
.highlight .c1 { color: #408080; font-style: italic } /* Comment.Single */
.highlight .cs { color: #408080; font-style: italic } /* Comment.Special */
.highlight .gd { color: #A00000 } /* Generic.Deleted */
.highlight .ge { font-style: italic } /* Generic.Emph */
.highlight .gr { color: #FF0000 } /* Generic.Error */
.highlight .gh { color: #000080; font-weight: bold } /* Generic.Heading */
.highlight .gi { color: #00A000 } /* Generic.Inserted */
.highlight .go { color: #808080 } /* Generic.Output */
.highlight .gp { color: #000080; font-weight: bold } /* Generic.Prompt */
.highlight .gs { font-weight: bold } /* Generic.Strong */
.highlight .gu { color: #800080; font-weight: bold } /* Generic.Subheading */
.highlight .gt { color: #0040D0 } /* Generic.Traceback */
.highlight .kc { color: #008000; font-weight: bold } /* Keyword.Constant */
.highlight .kd { color: #008000; font-weight: bold } /* Keyword.Declaration */
.highlight .kn { color: #008000; font-weight: bold } /* Keyword.Namespace */
.highlight .kp { color: #008000 } /* Keyword.Pseudo */
.highlight .kr { color: #008000; font-weight: bold } /* Keyword.Reserved */
.highlight .kt { color: #B00040 } /* Keyword.Type */
.highlight .m { color: #666666 } /* Literal.Number */
.highlight .s { color: #BA2121 } /* Literal.String */
.highlight .na { color: #7D9029 } /* Name.Attribute */
.highlight .nb { color: #008000 } /* Name.Builtin */
.highlight .nc { color: #0000FF; font-weight: bold } /* Name.Class */
.highlight .no { color: #880000 } /* Name.Constant */
.highlight .nd { color: #AA22FF } /* Name.Decorator */
.highlight .ni { color: #999999; font-weight: bold } /* Name.Entity */
.highlight .ne { color: #D2413A; font-weight: bold } /* Name.Exception */
.highlight .nf { color: #0000FF } /* Name.Function */
.highlight .nl { color: #A0A000 } /* Name.Label */
.highlight .nn { color: #0000FF; font-weight: bold } /* Name.Namespace */
.highlight .nt { color: #008000; font-weight: bold } /* Name.Tag */
.highlight .nv { color: #19177C } /* Name.Variable */
.highlight .ow { color: #AA22FF; font-weight: bold } /* Operator.Word */
.highlight .w { color: #bbbbbb } /* Text.Whitespace */
.highlight .mf { color: #666666 } /* Literal.Number.Float */
.highlight .mh { color: #666666 } /* Literal.Number.Hex */
.highlight .mi { color: #666666 } /* Literal.Number.Integer */
.highlight .mo { color: #666666 } /* Literal.Number.Oct */
.highlight .sb { color: #BA2121 } /* Literal.String.Backtick */
.highlight .sc { color: #BA2121 } /* Literal.String.Char */
.highlight .sd { color: #BA2121; font-style: italic } /* Literal.String.Doc */
.highlight .s2 { color: #BA2121 } /* Literal.String.Double */
.highlight .se { color: #BB6622; font-weight: bold } /* Literal.String.Escape */
.highlight .sh { color: #BA2121 } /* Literal.String.Heredoc */
.highlight .si { color: #BB6688; font-weight: bold } /* Literal.String.Interpol */
.highlight .sx { color: #008000 } /* Literal.String.Other */
.highlight .sr { color: #BB6688 } /* Literal.String.Regex */
.highlight .s1 { color: #BA2121 } /* Literal.String.Single */
.highlight .ss { color: #19177C } /* Literal.String.Symbol */
.highlight .bp { color: #008000 } /* Name.Builtin.Pseudo */
.highlight .vc { color: #19177C } /* Name.Variable.Class */
.highlight .vg { color: #19177C } /* Name.Variable.Global */
.highlight .vi { color: #19177C } /* Name.Variable.Instance */
.highlight .il { color: #666666 } /* Literal.Number.Integer.Long */


[data-bs-theme=dark] {
  td.linenos { background-color: #292929;}
  span.lineno { background-color: #292929; }
  .highlight .hll { background-color: #262800 }
  .highlight .c { color: #69a9a9; } /* Comment */
  .highlight .err { border-color: #ff2d13 } /* Error */
  .highlight .k { color: #56bb45 } /* Keyword */
  .highlight .o { color: #aeaeae } /* Operator */
  .highlight .cm { color: #69a9a9 } /* Comment.Multiline */
  .highlight .cp { color: #BC7A00 } /* Comment.Preproc */
  .highlight .c1 { color: #69a9a9 } /* Comment.Single */
  .highlight .cs { color: #69a9a9 } /* Comment.Special */
  .highlight .gd { color: #ff9871 } /* Generic.Deleted */
  .highlight .gr { color: #ff2d13 } /* Generic.Error */
  .highlight .gh { color: #ffeeff } /* Generic.Heading */
  .highlight .gi { color: #00A000 } /* Generic.Inserted */
  .highlight .go { color: #939393 } /* Generic.Output */
  .highlight .gp { color: #ffeeff } /* Generic.Prompt */
  .highlight .gu { color: #ffa9ff } /* Generic.Subheading */
  .highlight .gt { color: #bcbaff } /* Generic.Traceback */
  .highlight .kc { color: #56bb45 } /* Keyword.Constant */
  .highlight .kd { color: #56bb45 } /* Keyword.Declaration */
  .highlight .kn { color: #56bb45 } /* Keyword.Namespace */
  .highlight .kp { color: #56bb45 } /* Keyword.Pseudo */
  .highlight .kr { color: #56bb45 } /* Keyword.Reserved */
  .highlight .kt { color: #ff879f } /* Keyword.Type */
  .highlight .m { color: #aeaeae } /* Literal.Number */
  .highlight .s { color: #ff806b } /* Literal.String */
  .highlight .na { color: #7f922b } /* Name.Attribute */
  .highlight .nb { color: #56bb45 } /* Name.Builtin */
  .highlight .nc { color: #eba5ff } /* Name.Class */
  .highlight .no { color: #ffae8c } /* Name.Constant */
  .highlight .nd { color: #e765ff } /* Name.Decorator */
  .highlight .ni { color: #7a7a7a } /* Name.Entity */
  .highlight .ne { color: #ff6a5c } /* Name.Exception */
  .highlight .nf { color: #eba5ff } /* Name.Function */
  .highlight .nl { color: #797c00 } /* Name.Label */
  .highlight .nn { color: #eba5ff } /* Name.Namespace */
  .highlight .nt { color: #56bb45 } /* Name.Tag */
  .highlight .nv { color: #ffe9ff } /* Name.Variable */
  .highlight .ow { color: #e765ff } /* Operator.Word */
  .highlight .w { color: #5b5b5b } /* Text.Whitespace */
  .highlight .mf { color: #aeaeae } /* Literal.Number.Float */
  .highlight .mh { color: #aeaeae } /* Literal.Number.Hex */
  .highlight .mi { color: #aeaeae } /* Literal.Number.Integer */
  .highlight .mo { color: #aeaeae } /* Literal.Number.Oct */
  .highlight .sb { color: #ff806b } /* Literal.String.Backtick */
  .highlight .sc { color: #ff806b } /* Literal.String.Char */
  .highlight .sd { color: #ff806b } /* Literal.String.Doc */
  .highlight .s2 { color: #ff806b } /* Literal.String.Double */
  .highlight .se { color: #d77e39 } /* Literal.String.Escape */
  .highlight .sh { color: #ff806b } /* Literal.String.Heredoc */
  .highlight .si { color: #cd7698 } /* Literal.String.Interpol */
  .highlight .sx { color: #56bb45 } /* Literal.String.Other */
  .highlight .sr { color: #cd7698 } /* Literal.String.Regex */
  .highlight .s1 { color: #ff806b } /* Literal.String.Single */
  .highlight .ss { color: #ffe9ff } /* Literal.String.Symbol */
  .highlight .bp { color: #56bb45 } /* Name.Builtin.Pseudo */
  .highlight .vc { color: #ffe9ff } /* Name.Variable.Class */
  .highlight .vg { color: #ffe9ff } /* Name.Variable.Global */
  .highlight .vi { color: #ffe9ff } /* Name.Variable.Instance */
  .highlight .il { color: #aeaeae } /* Literal.Number.Integer.Long */
}