.yellow-box {
    background-color: #ffffb4;
    border: 1px solid black;
    margin-top: 1ex;
    margin-bottom: 1ex;
}

.green-box {
    background-color: #c4ffb4;
    min-height: 30px;
    text-align: center;
    padding: 3px;
}

.blue-box {
    background-color: #65dae8;
    min-height: 30px;
    text-align: center;
    padding: 3px;
}

[data-bs-theme=dark] {
    .yellow-box {
        background-color: #8b7500;
        border-color: white;
    }

    .green-box {
        background-color: #003a00;
    }

    .blue-box {
        background-color: #005c6a;
    }
}