/*
 * Copyright (C) 2025 Emweb bv, Herent, Belgium.
 *
 * See the LICENSE file for terms of use.
 */

#include <Wt/WContainerWidget.h>

class FormatExample : public Wt::WContainerWidget
{
public:
FormatExample();
};