.Wt-tree {
  cursor: default;

  ul {
    padding-inline-start: 20px;
    margin: 0;
    list-style: none;
  }

  div.Wt-root {
    display: none;
  }

  ul.Wt-root {
    padding-inline-start: 0;
  }

  &.Wt-trunk,
  .Wt-item.Wt-trunk {
    background-image: url(#{$wt-resources-prefix}line-trunk.gif);
    background-repeat: repeat-y;
  }

  .Wt-item {
    // Star character?
    *zoom: 1;

    &.Wt-end {
      background-image: url(#{$wt-resources-prefix}line-last.gif);
      background-position: 0 0;
      background-repeat: no-repeat;
    }
  }

  .active {
    color: $pagination-bg;
    text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.2);
    cursor: default;
    background-color: #428bca;
  }

  .Wt-ctrl {
    display: block;
    float: inline-start;
    width: 19px;
    height: 19px;

    &.Wt-expand {
      background: url(#{$wt-theme-prefix}nav-minus.gif) no-repeat 0 center;

      &.Wt-collapsed {
        background: url(#{$wt-theme-prefix}nav-plus.gif) no-repeat 0 center;
      }
    }


    &.Wt-noexpand {
      background: url(#{$wt-resources-prefix}line-middle.gif) no-repeat 0 0;
    }

    img {
      vertical-align: top;
      width: 19px;
      height: 19px;
    }
  }

  .cols-row {
    float: inline-end;
  }

  .Wt-label {
    padding: 0 2px;
    line-height: 150%;
  }
}


/*rtl:begin:ignore*/
body.Wt-rtl {
  .Wt-tree {

    &.Wt-trunk,
    .Wt-item.Wt-trunk {
      background-position: top right;
    }

    .Wt-item.Wt-end {
      background-image: url(#{$wt-resources-prefix}line-last-rtl.gif);
      background-position: top right;
    }

    .Wt-ctrl {
      &.Wt-expand {
      background: url(#{$wt-theme-prefix}nav-minus-rtl.gif) no-repeat 0 center;

      &.Wt-collapsed {
        background: url(#{$wt-theme-prefix}nav-plus-rtl.gif) no-repeat 0 center;
      }
    }

      &.Wt-noexpand {
        background-image: url(#{$wt-resources-prefix}line-middle-rtl.gif);
        background-position: top right;
      }
    }
  }
}

@include color-mode(dark) {
  .Wt-tree {
    &.Wt-trunk,
    .Wt-item.Wt-trunk {
      background-image: url(#{$wt-resources-prefix}line-trunk-dark.gif);
    }

    .Wt-item.Wt-end {
      background-image: url(#{$wt-resources-prefix}line-last-dark.gif);
    }

    .active {
      text-shadow: 0 -1px 0 rgba(255, 255, 255, 0.4);
      background-color: #4d94d4;
    }

    .Wt-ctrl {
      &.Wt-expand {
        background-image: url(#{$wt-theme-prefix}nav-minus-dark.gif);

        &.Wt-collapsed {
          background-image: url(#{$wt-theme-prefix}nav-plus-dark.gif);
        }
      }

      &.Wt-noexpand {
        background-image: url(#{$wt-resources-prefix}line-middle-dark.gif);
      }
    }
  }

  body.Wt-rtl {
    .Wt-tree {
      .Wt-item.Wt-end {
        background-image: url(#{$wt-resources-prefix}line-last-dark-rtl.gif);
      }

      .Wt-ctrl{
        &.Wt-expand {
          background-image: url(#{$wt-theme-prefix}nav-minus-dark-rtl.gif);

          &.Wt-collapsed {
            background-image: url(#{$wt-theme-prefix}nav-plus-dark-rtl.gif);
          }
        }

        &.Wt-noexpand {
          background-image: url(#{$wt-resources-prefix}line-middle-dark-rtl.gif);
        }
      }
    }
  }
}

/*rtl:end:ignore*/
