.upload-block {
    width: 10px;
    height: 10px;
    margin: 10px;
    border: 1px solid black;
    display: inline-block;
}

.upload-block.ready {
    background-color: green;
}

.upload-block.failed {
    background-color: red;
}

/* spinner styling from http://tobiasahlin.com/spinkit/ */
.spinner {
  width: 10px;
  height: 10px;
  background-color: #333;

  border-radius: 100%;  
  -webkit-animation: sk-scaleout 1.0s infinite ease-in-out;
  animation: sk-scaleout 1.0s infinite ease-in-out;
}

@-webkit-keyframes sk-scaleout {
  0% { -webkit-transform: scale(0) }
  100% {
    -webkit-transform: scale(1.0);
    opacity: 0;
  }
}

@keyframes sk-scaleout {
  0% {
    -webkit-transform: scale(0);
    transform: scale(0);
  } 100% {
    -webkit-transform: scale(1.0);
    transform: scale(1.0);
    opacity: 0;
  }